@@ -26,7 +26,9 @@ __all__ = (
     "ServiceUpdateStatusType",
     "ServiceUpdateTypeType",
     "SourceTypeType",
+    "MemoryDBServiceName",
     "ServiceName",
+    "ResourceServiceName",
 )
 
 
@@ -36,6 +38,7 @@ InputAuthenticationTypeType = Literal["password"]
 ServiceUpdateStatusType = Literal["available", "complete", "in-progress", "scheduled"]
 ServiceUpdateTypeType = Literal["security-update"]
 SourceTypeType = Literal["acl", "cluster", "node", "parameter-group", "subnet-group", "user"]
+MemoryDBServiceName = Literal["memorydb"]
 ServiceName = Literal[
     "accessanalyzer",
     "account",
@@ -67,6 +70,7 @@ ServiceName = Literal[
     "backup",
     "backup-gateway",
     "batch",
+    "billingconductor",
     "braket",
     "budgets",
     "ce",
@@ -189,6 +193,7 @@ ServiceName = Literal[
     "kafka",
     "kafkaconnect",
     "kendra",
+    "keyspaces",
     "kinesis",
     "kinesis-video-archived-media",
     "kinesis-video-media",
@@ -337,3 +342,15 @@ ServiceName = Literal[
     "workspaces-web",
     "xray",
 ]
+ResourceServiceName = Literal[
+    "cloudformation",
+    "cloudwatch",
+    "dynamodb",
+    "ec2",
+    "glacier",
+    "iam",
+    "opsworks",
+    "s3",
+    "sns",
+    "sqs",
+]
