@@ -15,6 +15,8 @@ import sys
 from datetime import datetime
 from typing import Dict, List, Sequence, Union
 
+from typing_extensions import NotRequired
+
 from .literals import AuthenticationTypeType, AZStatusType, ServiceUpdateStatusType, SourceTypeType
 
 if sys.version_info >= (3, 9):
@@ -138,81 +140,61 @@ __all__ = (
 ACLPendingChangesTypeDef = TypedDict(
     "ACLPendingChangesTypeDef",
     {
-        "UserNamesToRemove": List[str],
-        "UserNamesToAdd": List[str],
+        "UserNamesToRemove": NotRequired[List[str]],
+        "UserNamesToAdd": NotRequired[List[str]],
     },
-    total=False,
 )
 
 ACLTypeDef = TypedDict(
     "ACLTypeDef",
     {
-        "Name": str,
-        "Status": str,
-        "UserNames": List[str],
-        "MinimumEngineVersion": str,
-        "PendingChanges": "ACLPendingChangesTypeDef",
-        "Clusters": List[str],
-        "ARN": str,
+        "Name": NotRequired[str],
+        "Status": NotRequired[str],
+        "UserNames": NotRequired[List[str]],
+        "MinimumEngineVersion": NotRequired[str],
+        "PendingChanges": NotRequired["ACLPendingChangesTypeDef"],
+        "Clusters": NotRequired[List[str]],
+        "ARN": NotRequired[str],
     },
-    total=False,
 )
 
 ACLsUpdateStatusTypeDef = TypedDict(
     "ACLsUpdateStatusTypeDef",
     {
-        "ACLToApply": str,
+        "ACLToApply": NotRequired[str],
     },
-    total=False,
 )
 
 AuthenticationModeTypeDef = TypedDict(
     "AuthenticationModeTypeDef",
     {
-        "Type": Literal["password"],
-        "Passwords": Sequence[str],
+        "Type": NotRequired[Literal["password"]],
+        "Passwords": NotRequired[Sequence[str]],
     },
-    total=False,
 )
 
 AuthenticationTypeDef = TypedDict(
     "AuthenticationTypeDef",
     {
-        "Type": AuthenticationTypeType,
-        "PasswordCount": int,
+        "Type": NotRequired[AuthenticationTypeType],
+        "PasswordCount": NotRequired[int],
     },
-    total=False,
 )
 
 AvailabilityZoneTypeDef = TypedDict(
     "AvailabilityZoneTypeDef",
     {
-        "Name": str,
+        "Name": NotRequired[str],
     },
-    total=False,
 )
 
-_RequiredBatchUpdateClusterRequestRequestTypeDef = TypedDict(
-    "_RequiredBatchUpdateClusterRequestRequestTypeDef",
+BatchUpdateClusterRequestRequestTypeDef = TypedDict(
+    "BatchUpdateClusterRequestRequestTypeDef",
     {
         "ClusterNames": Sequence[str],
+        "ServiceUpdate": NotRequired["ServiceUpdateRequestTypeDef"],
     },
 )
-_OptionalBatchUpdateClusterRequestRequestTypeDef = TypedDict(
-    "_OptionalBatchUpdateClusterRequestRequestTypeDef",
-    {
-        "ServiceUpdate": "ServiceUpdateRequestTypeDef",
-    },
-    total=False,
-)
-
-
-class BatchUpdateClusterRequestRequestTypeDef(
-    _RequiredBatchUpdateClusterRequestRequestTypeDef,
-    _OptionalBatchUpdateClusterRequestRequestTypeDef,
-):
-    pass
-
 
 BatchUpdateClusterResponseTypeDef = TypedDict(
     "BatchUpdateClusterResponseTypeDef",
@@ -226,89 +208,73 @@ BatchUpdateClusterResponseTypeDef = TypedDict(
 ClusterConfigurationTypeDef = TypedDict(
     "ClusterConfigurationTypeDef",
     {
-        "Name": str,
-        "Description": str,
-        "NodeType": str,
-        "EngineVersion": str,
-        "MaintenanceWindow": str,
-        "TopicArn": str,
-        "Port": int,
-        "ParameterGroupName": str,
-        "SubnetGroupName": str,
-        "VpcId": str,
-        "SnapshotRetentionLimit": int,
-        "SnapshotWindow": str,
-        "NumShards": int,
-        "Shards": List["ShardDetailTypeDef"],
+        "Name": NotRequired[str],
+        "Description": NotRequired[str],
+        "NodeType": NotRequired[str],
+        "EngineVersion": NotRequired[str],
+        "MaintenanceWindow": NotRequired[str],
+        "TopicArn": NotRequired[str],
+        "Port": NotRequired[int],
+        "ParameterGroupName": NotRequired[str],
+        "SubnetGroupName": NotRequired[str],
+        "VpcId": NotRequired[str],
+        "SnapshotRetentionLimit": NotRequired[int],
+        "SnapshotWindow": NotRequired[str],
+        "NumShards": NotRequired[int],
+        "Shards": NotRequired[List["ShardDetailTypeDef"]],
     },
-    total=False,
 )
 
 ClusterPendingUpdatesTypeDef = TypedDict(
     "ClusterPendingUpdatesTypeDef",
     {
-        "Resharding": "ReshardingStatusTypeDef",
-        "ACLs": "ACLsUpdateStatusTypeDef",
-        "ServiceUpdates": List["PendingModifiedServiceUpdateTypeDef"],
+        "Resharding": NotRequired["ReshardingStatusTypeDef"],
+        "ACLs": NotRequired["ACLsUpdateStatusTypeDef"],
+        "ServiceUpdates": NotRequired[List["PendingModifiedServiceUpdateTypeDef"]],
     },
-    total=False,
 )
 
 ClusterTypeDef = TypedDict(
     "ClusterTypeDef",
     {
-        "Name": str,
-        "Description": str,
-        "Status": str,
-        "PendingUpdates": "ClusterPendingUpdatesTypeDef",
-        "NumberOfShards": int,
-        "Shards": List["ShardTypeDef"],
-        "AvailabilityMode": AZStatusType,
-        "ClusterEndpoint": "EndpointTypeDef",
-        "NodeType": str,
-        "EngineVersion": str,
-        "EnginePatchVersion": str,
-        "ParameterGroupName": str,
-        "ParameterGroupStatus": str,
-        "SecurityGroups": List["SecurityGroupMembershipTypeDef"],
-        "SubnetGroupName": str,
-        "TLSEnabled": bool,
-        "KmsKeyId": str,
-        "ARN": str,
-        "SnsTopicArn": str,
-        "SnsTopicStatus": str,
-        "SnapshotRetentionLimit": int,
-        "MaintenanceWindow": str,
-        "SnapshotWindow": str,
-        "ACLName": str,
-        "AutoMinorVersionUpgrade": bool,
+        "Name": NotRequired[str],
+        "Description": NotRequired[str],
+        "Status": NotRequired[str],
+        "PendingUpdates": NotRequired["ClusterPendingUpdatesTypeDef"],
+        "NumberOfShards": NotRequired[int],
+        "Shards": NotRequired[List["ShardTypeDef"]],
+        "AvailabilityMode": NotRequired[AZStatusType],
+        "ClusterEndpoint": NotRequired["EndpointTypeDef"],
+        "NodeType": NotRequired[str],
+        "EngineVersion": NotRequired[str],
+        "EnginePatchVersion": NotRequired[str],
+        "ParameterGroupName": NotRequired[str],
+        "ParameterGroupStatus": NotRequired[str],
+        "SecurityGroups": NotRequired[List["SecurityGroupMembershipTypeDef"]],
+        "SubnetGroupName": NotRequired[str],
+        "TLSEnabled": NotRequired[bool],
+        "KmsKeyId": NotRequired[str],
+        "ARN": NotRequired[str],
+        "SnsTopicArn": NotRequired[str],
+        "SnsTopicStatus": NotRequired[str],
+        "SnapshotRetentionLimit": NotRequired[int],
+        "MaintenanceWindow": NotRequired[str],
+        "SnapshotWindow": NotRequired[str],
+        "ACLName": NotRequired[str],
+        "AutoMinorVersionUpgrade": NotRequired[bool],
     },
-    total=False,
 )
 
-_RequiredCopySnapshotRequestRequestTypeDef = TypedDict(
-    "_RequiredCopySnapshotRequestRequestTypeDef",
+CopySnapshotRequestRequestTypeDef = TypedDict(
+    "CopySnapshotRequestRequestTypeDef",
     {
         "SourceSnapshotName": str,
         "TargetSnapshotName": str,
+        "TargetBucket": NotRequired[str],
+        "KmsKeyId": NotRequired[str],
+        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
-_OptionalCopySnapshotRequestRequestTypeDef = TypedDict(
-    "_OptionalCopySnapshotRequestRequestTypeDef",
-    {
-        "TargetBucket": str,
-        "KmsKeyId": str,
-        "Tags": Sequence["TagTypeDef"],
-    },
-    total=False,
-)
-
-
-class CopySnapshotRequestRequestTypeDef(
-    _RequiredCopySnapshotRequestRequestTypeDef, _OptionalCopySnapshotRequestRequestTypeDef
-):
-    pass
-
 
 CopySnapshotResponseTypeDef = TypedDict(
     "CopySnapshotResponseTypeDef",
@@ -318,27 +284,14 @@ CopySnapshotResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredCreateACLRequestRequestTypeDef = TypedDict(
-    "_RequiredCreateACLRequestRequestTypeDef",
+CreateACLRequestRequestTypeDef = TypedDict(
+    "CreateACLRequestRequestTypeDef",
     {
         "ACLName": str,
+        "UserNames": NotRequired[Sequence[str]],
+        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
-_OptionalCreateACLRequestRequestTypeDef = TypedDict(
-    "_OptionalCreateACLRequestRequestTypeDef",
-    {
-        "UserNames": Sequence[str],
-        "Tags": Sequence["TagTypeDef"],
-    },
-    total=False,
-)
-
-
-class CreateACLRequestRequestTypeDef(
-    _RequiredCreateACLRequestRequestTypeDef, _OptionalCreateACLRequestRequestTypeDef
-):
-    pass
-
 
 CreateACLResponseTypeDef = TypedDict(
     "CreateACLResponseTypeDef",
@@ -348,45 +301,32 @@ CreateACLResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredCreateClusterRequestRequestTypeDef = TypedDict(
-    "_RequiredCreateClusterRequestRequestTypeDef",
+CreateClusterRequestRequestTypeDef = TypedDict(
+    "CreateClusterRequestRequestTypeDef",
     {
         "ClusterName": str,
         "NodeType": str,
         "ACLName": str,
+        "ParameterGroupName": NotRequired[str],
+        "Description": NotRequired[str],
+        "NumShards": NotRequired[int],
+        "NumReplicasPerShard": NotRequired[int],
+        "SubnetGroupName": NotRequired[str],
+        "SecurityGroupIds": NotRequired[Sequence[str]],
+        "MaintenanceWindow": NotRequired[str],
+        "Port": NotRequired[int],
+        "SnsTopicArn": NotRequired[str],
+        "TLSEnabled": NotRequired[bool],
+        "KmsKeyId": NotRequired[str],
+        "SnapshotArns": NotRequired[Sequence[str]],
+        "SnapshotName": NotRequired[str],
+        "SnapshotRetentionLimit": NotRequired[int],
+        "Tags": NotRequired[Sequence["TagTypeDef"]],
+        "SnapshotWindow": NotRequired[str],
+        "EngineVersion": NotRequired[str],
+        "AutoMinorVersionUpgrade": NotRequired[bool],
     },
 )
-_OptionalCreateClusterRequestRequestTypeDef = TypedDict(
-    "_OptionalCreateClusterRequestRequestTypeDef",
-    {
-        "ParameterGroupName": str,
-        "Description": str,
-        "NumShards": int,
-        "NumReplicasPerShard": int,
-        "SubnetGroupName": str,
-        "SecurityGroupIds": Sequence[str],
-        "MaintenanceWindow": str,
-        "Port": int,
-        "SnsTopicArn": str,
-        "TLSEnabled": bool,
-        "KmsKeyId": str,
-        "SnapshotArns": Sequence[str],
-        "SnapshotName": str,
-        "SnapshotRetentionLimit": int,
-        "Tags": Sequence["TagTypeDef"],
-        "SnapshotWindow": str,
-        "EngineVersion": str,
-        "AutoMinorVersionUpgrade": bool,
-    },
-    total=False,
-)
-
-
-class CreateClusterRequestRequestTypeDef(
-    _RequiredCreateClusterRequestRequestTypeDef, _OptionalCreateClusterRequestRequestTypeDef
-):
-    pass
-
 
 CreateClusterResponseTypeDef = TypedDict(
     "CreateClusterResponseTypeDef",
@@ -396,29 +336,15 @@ CreateClusterResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredCreateParameterGroupRequestRequestTypeDef = TypedDict(
-    "_RequiredCreateParameterGroupRequestRequestTypeDef",
+CreateParameterGroupRequestRequestTypeDef = TypedDict(
+    "CreateParameterGroupRequestRequestTypeDef",
     {
         "ParameterGroupName": str,
         "Family": str,
+        "Description": NotRequired[str],
+        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
-_OptionalCreateParameterGroupRequestRequestTypeDef = TypedDict(
-    "_OptionalCreateParameterGroupRequestRequestTypeDef",
-    {
-        "Description": str,
-        "Tags": Sequence["TagTypeDef"],
-    },
-    total=False,
-)
-
-
-class CreateParameterGroupRequestRequestTypeDef(
-    _RequiredCreateParameterGroupRequestRequestTypeDef,
-    _OptionalCreateParameterGroupRequestRequestTypeDef,
-):
-    pass
-
 
 CreateParameterGroupResponseTypeDef = TypedDict(
     "CreateParameterGroupResponseTypeDef",
@@ -428,28 +354,15 @@ CreateParameterGroupResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredCreateSnapshotRequestRequestTypeDef = TypedDict(
-    "_RequiredCreateSnapshotRequestRequestTypeDef",
+CreateSnapshotRequestRequestTypeDef = TypedDict(
+    "CreateSnapshotRequestRequestTypeDef",
     {
         "ClusterName": str,
         "SnapshotName": str,
+        "KmsKeyId": NotRequired[str],
+        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
-_OptionalCreateSnapshotRequestRequestTypeDef = TypedDict(
-    "_OptionalCreateSnapshotRequestRequestTypeDef",
-    {
-        "KmsKeyId": str,
-        "Tags": Sequence["TagTypeDef"],
-    },
-    total=False,
-)
-
-
-class CreateSnapshotRequestRequestTypeDef(
-    _RequiredCreateSnapshotRequestRequestTypeDef, _OptionalCreateSnapshotRequestRequestTypeDef
-):
-    pass
-
 
 CreateSnapshotResponseTypeDef = TypedDict(
     "CreateSnapshotResponseTypeDef",
@@ -459,28 +372,15 @@ CreateSnapshotResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredCreateSubnetGroupRequestRequestTypeDef = TypedDict(
-    "_RequiredCreateSubnetGroupRequestRequestTypeDef",
+CreateSubnetGroupRequestRequestTypeDef = TypedDict(
+    "CreateSubnetGroupRequestRequestTypeDef",
     {
         "SubnetGroupName": str,
         "SubnetIds": Sequence[str],
+        "Description": NotRequired[str],
+        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
-_OptionalCreateSubnetGroupRequestRequestTypeDef = TypedDict(
-    "_OptionalCreateSubnetGroupRequestRequestTypeDef",
-    {
-        "Description": str,
-        "Tags": Sequence["TagTypeDef"],
-    },
-    total=False,
-)
-
-
-class CreateSubnetGroupRequestRequestTypeDef(
-    _RequiredCreateSubnetGroupRequestRequestTypeDef, _OptionalCreateSubnetGroupRequestRequestTypeDef
-):
-    pass
-
 
 CreateSubnetGroupResponseTypeDef = TypedDict(
     "CreateSubnetGroupResponseTypeDef",
@@ -490,28 +390,15 @@ CreateSubnetGroupResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredCreateUserRequestRequestTypeDef = TypedDict(
-    "_RequiredCreateUserRequestRequestTypeDef",
+CreateUserRequestRequestTypeDef = TypedDict(
+    "CreateUserRequestRequestTypeDef",
     {
         "UserName": str,
         "AuthenticationMode": "AuthenticationModeTypeDef",
         "AccessString": str,
+        "Tags": NotRequired[Sequence["TagTypeDef"]],
     },
 )
-_OptionalCreateUserRequestRequestTypeDef = TypedDict(
-    "_OptionalCreateUserRequestRequestTypeDef",
-    {
-        "Tags": Sequence["TagTypeDef"],
-    },
-    total=False,
-)
-
-
-class CreateUserRequestRequestTypeDef(
-    _RequiredCreateUserRequestRequestTypeDef, _OptionalCreateUserRequestRequestTypeDef
-):
-    pass
-
 
 CreateUserResponseTypeDef = TypedDict(
     "CreateUserResponseTypeDef",
@@ -536,26 +423,13 @@ DeleteACLResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredDeleteClusterRequestRequestTypeDef = TypedDict(
-    "_RequiredDeleteClusterRequestRequestTypeDef",
+DeleteClusterRequestRequestTypeDef = TypedDict(
+    "DeleteClusterRequestRequestTypeDef",
     {
         "ClusterName": str,
+        "FinalSnapshotName": NotRequired[str],
     },
 )
-_OptionalDeleteClusterRequestRequestTypeDef = TypedDict(
-    "_OptionalDeleteClusterRequestRequestTypeDef",
-    {
-        "FinalSnapshotName": str,
-    },
-    total=False,
-)
-
-
-class DeleteClusterRequestRequestTypeDef(
-    _RequiredDeleteClusterRequestRequestTypeDef, _OptionalDeleteClusterRequestRequestTypeDef
-):
-    pass
-
 
 DeleteClusterResponseTypeDef = TypedDict(
     "DeleteClusterResponseTypeDef",
@@ -628,11 +502,10 @@ DeleteUserResponseTypeDef = TypedDict(
 DescribeACLsRequestRequestTypeDef = TypedDict(
     "DescribeACLsRequestRequestTypeDef",
     {
-        "ACLName": str,
-        "MaxResults": int,
-        "NextToken": str,
+        "ACLName": NotRequired[str],
+        "MaxResults": NotRequired[int],
+        "NextToken": NotRequired[str],
     },
-    total=False,
 )
 
 DescribeACLsResponseTypeDef = TypedDict(
@@ -647,12 +520,11 @@ DescribeACLsResponseTypeDef = TypedDict(
 DescribeClustersRequestRequestTypeDef = TypedDict(
     "DescribeClustersRequestRequestTypeDef",
     {
-        "ClusterName": str,
-        "MaxResults": int,
-        "NextToken": str,
-        "ShowShardDetails": bool,
+        "ClusterName": NotRequired[str],
+        "MaxResults": NotRequired[int],
+        "NextToken": NotRequired[str],
+        "ShowShardDetails": NotRequired[bool],
     },
-    total=False,
 )
 
 DescribeClustersResponseTypeDef = TypedDict(
@@ -667,13 +539,12 @@ DescribeClustersResponseTypeDef = TypedDict(
 DescribeEngineVersionsRequestRequestTypeDef = TypedDict(
     "DescribeEngineVersionsRequestRequestTypeDef",
     {
-        "EngineVersion": str,
-        "ParameterGroupFamily": str,
-        "MaxResults": int,
-        "NextToken": str,
-        "DefaultOnly": bool,
+        "EngineVersion": NotRequired[str],
+        "ParameterGroupFamily": NotRequired[str],
+        "MaxResults": NotRequired[int],
+        "NextToken": NotRequired[str],
+        "DefaultOnly": NotRequired[bool],
     },
-    total=False,
 )
 
 DescribeEngineVersionsResponseTypeDef = TypedDict(
@@ -688,15 +559,14 @@ DescribeEngineVersionsResponseTypeDef = TypedDict(
 DescribeEventsRequestRequestTypeDef = TypedDict(
     "DescribeEventsRequestRequestTypeDef",
     {
-        "SourceName": str,
-        "SourceType": SourceTypeType,
-        "StartTime": Union[datetime, str],
-        "EndTime": Union[datetime, str],
-        "Duration": int,
-        "MaxResults": int,
-        "NextToken": str,
+        "SourceName": NotRequired[str],
+        "SourceType": NotRequired[SourceTypeType],
+        "StartTime": NotRequired[Union[datetime, str]],
+        "EndTime": NotRequired[Union[datetime, str]],
+        "Duration": NotRequired[int],
+        "MaxResults": NotRequired[int],
+        "NextToken": NotRequired[str],
     },
-    total=False,
 )
 
 DescribeEventsResponseTypeDef = TypedDict(
@@ -711,11 +581,10 @@ DescribeEventsResponseTypeDef = TypedDict(
 DescribeParameterGroupsRequestRequestTypeDef = TypedDict(
     "DescribeParameterGroupsRequestRequestTypeDef",
     {
-        "ParameterGroupName": str,
-        "MaxResults": int,
-        "NextToken": str,
+        "ParameterGroupName": NotRequired[str],
+        "MaxResults": NotRequired[int],
+        "NextToken": NotRequired[str],
     },
-    total=False,
 )
 
 DescribeParameterGroupsResponseTypeDef = TypedDict(
@@ -727,28 +596,14 @@ DescribeParameterGroupsResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredDescribeParametersRequestRequestTypeDef = TypedDict(
-    "_RequiredDescribeParametersRequestRequestTypeDef",
+DescribeParametersRequestRequestTypeDef = TypedDict(
+    "DescribeParametersRequestRequestTypeDef",
     {
         "ParameterGroupName": str,
+        "MaxResults": NotRequired[int],
+        "NextToken": NotRequired[str],
     },
 )
-_OptionalDescribeParametersRequestRequestTypeDef = TypedDict(
-    "_OptionalDescribeParametersRequestRequestTypeDef",
-    {
-        "MaxResults": int,
-        "NextToken": str,
-    },
-    total=False,
-)
-
-
-class DescribeParametersRequestRequestTypeDef(
-    _RequiredDescribeParametersRequestRequestTypeDef,
-    _OptionalDescribeParametersRequestRequestTypeDef,
-):
-    pass
-
 
 DescribeParametersResponseTypeDef = TypedDict(
     "DescribeParametersResponseTypeDef",
@@ -762,13 +617,12 @@ DescribeParametersResponseTypeDef = TypedDict(
 DescribeServiceUpdatesRequestRequestTypeDef = TypedDict(
     "DescribeServiceUpdatesRequestRequestTypeDef",
     {
-        "ServiceUpdateName": str,
-        "ClusterNames": Sequence[str],
-        "Status": Sequence[ServiceUpdateStatusType],
-        "MaxResults": int,
-        "NextToken": str,
+        "ServiceUpdateName": NotRequired[str],
+        "ClusterNames": NotRequired[Sequence[str]],
+        "Status": NotRequired[Sequence[ServiceUpdateStatusType]],
+        "MaxResults": NotRequired[int],
+        "NextToken": NotRequired[str],
     },
-    total=False,
 )
 
 DescribeServiceUpdatesResponseTypeDef = TypedDict(
@@ -783,14 +637,13 @@ DescribeServiceUpdatesResponseTypeDef = TypedDict(
 DescribeSnapshotsRequestRequestTypeDef = TypedDict(
     "DescribeSnapshotsRequestRequestTypeDef",
     {
-        "ClusterName": str,
-        "SnapshotName": str,
-        "Source": str,
-        "NextToken": str,
-        "MaxResults": int,
-        "ShowDetail": bool,
+        "ClusterName": NotRequired[str],
+        "SnapshotName": NotRequired[str],
+        "Source": NotRequired[str],
+        "NextToken": NotRequired[str],
+        "MaxResults": NotRequired[int],
+        "ShowDetail": NotRequired[bool],
     },
-    total=False,
 )
 
 DescribeSnapshotsResponseTypeDef = TypedDict(
@@ -805,11 +658,10 @@ DescribeSnapshotsResponseTypeDef = TypedDict(
 DescribeSubnetGroupsRequestRequestTypeDef = TypedDict(
     "DescribeSubnetGroupsRequestRequestTypeDef",
     {
-        "SubnetGroupName": str,
-        "MaxResults": int,
-        "NextToken": str,
+        "SubnetGroupName": NotRequired[str],
+        "MaxResults": NotRequired[int],
+        "NextToken": NotRequired[str],
     },
-    total=False,
 )
 
 DescribeSubnetGroupsResponseTypeDef = TypedDict(
@@ -824,12 +676,11 @@ DescribeSubnetGroupsResponseTypeDef = TypedDict(
 DescribeUsersRequestRequestTypeDef = TypedDict(
     "DescribeUsersRequestRequestTypeDef",
     {
-        "UserName": str,
-        "Filters": Sequence["FilterTypeDef"],
-        "MaxResults": int,
-        "NextToken": str,
+        "UserName": NotRequired[str],
+        "Filters": NotRequired[Sequence["FilterTypeDef"]],
+        "MaxResults": NotRequired[int],
+        "NextToken": NotRequired[str],
     },
-    total=False,
 )
 
 DescribeUsersResponseTypeDef = TypedDict(
@@ -844,31 +695,28 @@ DescribeUsersResponseTypeDef = TypedDict(
 EndpointTypeDef = TypedDict(
     "EndpointTypeDef",
     {
-        "Address": str,
-        "Port": int,
+        "Address": NotRequired[str],
+        "Port": NotRequired[int],
     },
-    total=False,
 )
 
 EngineVersionInfoTypeDef = TypedDict(
     "EngineVersionInfoTypeDef",
     {
-        "EngineVersion": str,
-        "EnginePatchVersion": str,
-        "ParameterGroupFamily": str,
+        "EngineVersion": NotRequired[str],
+        "EnginePatchVersion": NotRequired[str],
+        "ParameterGroupFamily": NotRequired[str],
     },
-    total=False,
 )
 
 EventTypeDef = TypedDict(
     "EventTypeDef",
     {
-        "SourceName": str,
-        "SourceType": SourceTypeType,
-        "Message": str,
-        "Date": datetime,
+        "SourceName": NotRequired[str],
+        "SourceType": NotRequired[SourceTypeType],
+        "Message": NotRequired[str],
+        "Date": NotRequired[datetime],
     },
-    total=False,
 )
 
 FailoverShardRequestRequestTypeDef = TypedDict(
@@ -929,87 +777,67 @@ ListTagsResponseTypeDef = TypedDict(
 NodeTypeDef = TypedDict(
     "NodeTypeDef",
     {
-        "Name": str,
-        "Status": str,
-        "AvailabilityZone": str,
-        "CreateTime": datetime,
-        "Endpoint": "EndpointTypeDef",
+        "Name": NotRequired[str],
+        "Status": NotRequired[str],
+        "AvailabilityZone": NotRequired[str],
+        "CreateTime": NotRequired[datetime],
+        "Endpoint": NotRequired["EndpointTypeDef"],
     },
-    total=False,
 )
 
 ParameterGroupTypeDef = TypedDict(
     "ParameterGroupTypeDef",
     {
-        "Name": str,
-        "Family": str,
-        "Description": str,
-        "ARN": str,
+        "Name": NotRequired[str],
+        "Family": NotRequired[str],
+        "Description": NotRequired[str],
+        "ARN": NotRequired[str],
     },
-    total=False,
 )
 
 ParameterNameValueTypeDef = TypedDict(
     "ParameterNameValueTypeDef",
     {
-        "ParameterName": str,
-        "ParameterValue": str,
+        "ParameterName": NotRequired[str],
+        "ParameterValue": NotRequired[str],
     },
-    total=False,
 )
 
 ParameterTypeDef = TypedDict(
     "ParameterTypeDef",
     {
-        "Name": str,
-        "Value": str,
-        "Description": str,
-        "DataType": str,
-        "AllowedValues": str,
-        "MinimumEngineVersion": str,
+        "Name": NotRequired[str],
+        "Value": NotRequired[str],
+        "Description": NotRequired[str],
+        "DataType": NotRequired[str],
+        "AllowedValues": NotRequired[str],
+        "MinimumEngineVersion": NotRequired[str],
     },
-    total=False,
 )
 
 PendingModifiedServiceUpdateTypeDef = TypedDict(
     "PendingModifiedServiceUpdateTypeDef",
     {
-        "ServiceUpdateName": str,
-        "Status": ServiceUpdateStatusType,
+        "ServiceUpdateName": NotRequired[str],
+        "Status": NotRequired[ServiceUpdateStatusType],
     },
-    total=False,
 )
 
 ReplicaConfigurationRequestTypeDef = TypedDict(
     "ReplicaConfigurationRequestTypeDef",
     {
-        "ReplicaCount": int,
+        "ReplicaCount": NotRequired[int],
     },
-    total=False,
 )
 
-_RequiredResetParameterGroupRequestRequestTypeDef = TypedDict(
-    "_RequiredResetParameterGroupRequestRequestTypeDef",
+ResetParameterGroupRequestRequestTypeDef = TypedDict(
+    "ResetParameterGroupRequestRequestTypeDef",
     {
         "ParameterGroupName": str,
+        "AllParameters": NotRequired[bool],
+        "ParameterNames": NotRequired[Sequence[str]],
     },
 )
-_OptionalResetParameterGroupRequestRequestTypeDef = TypedDict(
-    "_OptionalResetParameterGroupRequestRequestTypeDef",
-    {
-        "AllParameters": bool,
-        "ParameterNames": Sequence[str],
-    },
-    total=False,
-)
-
-
-class ResetParameterGroupRequestRequestTypeDef(
-    _RequiredResetParameterGroupRequestRequestTypeDef,
-    _OptionalResetParameterGroupRequestRequestTypeDef,
-):
-    pass
-
 
 ResetParameterGroupResponseTypeDef = TypedDict(
     "ResetParameterGroupResponseTypeDef",
@@ -1022,9 +850,8 @@ ResetParameterGroupResponseTypeDef = TypedDict(
 ReshardingStatusTypeDef = TypedDict(
     "ReshardingStatusTypeDef",
     {
-        "SlotMigration": "SlotMigrationTypeDef",
+        "SlotMigration": NotRequired["SlotMigrationTypeDef"],
     },
-    total=False,
 )
 
 ResponseMetadataTypeDef = TypedDict(
@@ -1041,115 +868,104 @@ ResponseMetadataTypeDef = TypedDict(
 SecurityGroupMembershipTypeDef = TypedDict(
     "SecurityGroupMembershipTypeDef",
     {
-        "SecurityGroupId": str,
-        "Status": str,
+        "SecurityGroupId": NotRequired[str],
+        "Status": NotRequired[str],
     },
-    total=False,
 )
 
 ServiceUpdateRequestTypeDef = TypedDict(
     "ServiceUpdateRequestTypeDef",
     {
-        "ServiceUpdateNameToApply": str,
+        "ServiceUpdateNameToApply": NotRequired[str],
     },
-    total=False,
 )
 
 ServiceUpdateTypeDef = TypedDict(
     "ServiceUpdateTypeDef",
     {
-        "ClusterName": str,
-        "ServiceUpdateName": str,
-        "ReleaseDate": datetime,
-        "Description": str,
-        "Status": ServiceUpdateStatusType,
-        "Type": Literal["security-update"],
-        "NodesUpdated": str,
-        "AutoUpdateStartDate": datetime,
+        "ClusterName": NotRequired[str],
+        "ServiceUpdateName": NotRequired[str],
+        "ReleaseDate": NotRequired[datetime],
+        "Description": NotRequired[str],
+        "Status": NotRequired[ServiceUpdateStatusType],
+        "Type": NotRequired[Literal["security-update"]],
+        "NodesUpdated": NotRequired[str],
+        "AutoUpdateStartDate": NotRequired[datetime],
     },
-    total=False,
 )
 
 ShardConfigurationRequestTypeDef = TypedDict(
     "ShardConfigurationRequestTypeDef",
     {
-        "ShardCount": int,
+        "ShardCount": NotRequired[int],
     },
-    total=False,
 )
 
 ShardConfigurationTypeDef = TypedDict(
     "ShardConfigurationTypeDef",
     {
-        "Slots": str,
-        "ReplicaCount": int,
+        "Slots": NotRequired[str],
+        "ReplicaCount": NotRequired[int],
     },
-    total=False,
 )
 
 ShardDetailTypeDef = TypedDict(
     "ShardDetailTypeDef",
     {
-        "Name": str,
-        "Configuration": "ShardConfigurationTypeDef",
-        "Size": str,
-        "SnapshotCreationTime": datetime,
+        "Name": NotRequired[str],
+        "Configuration": NotRequired["ShardConfigurationTypeDef"],
+        "Size": NotRequired[str],
+        "SnapshotCreationTime": NotRequired[datetime],
     },
-    total=False,
 )
 
 ShardTypeDef = TypedDict(
     "ShardTypeDef",
     {
-        "Name": str,
-        "Status": str,
-        "Slots": str,
-        "Nodes": List["NodeTypeDef"],
-        "NumberOfNodes": int,
+        "Name": NotRequired[str],
+        "Status": NotRequired[str],
+        "Slots": NotRequired[str],
+        "Nodes": NotRequired[List["NodeTypeDef"]],
+        "NumberOfNodes": NotRequired[int],
     },
-    total=False,
 )
 
 SlotMigrationTypeDef = TypedDict(
     "SlotMigrationTypeDef",
     {
-        "ProgressPercentage": float,
+        "ProgressPercentage": NotRequired[float],
     },
-    total=False,
 )
 
 SnapshotTypeDef = TypedDict(
     "SnapshotTypeDef",
     {
-        "Name": str,
-        "Status": str,
-        "Source": str,
-        "KmsKeyId": str,
-        "ARN": str,
-        "ClusterConfiguration": "ClusterConfigurationTypeDef",
+        "Name": NotRequired[str],
+        "Status": NotRequired[str],
+        "Source": NotRequired[str],
+        "KmsKeyId": NotRequired[str],
+        "ARN": NotRequired[str],
+        "ClusterConfiguration": NotRequired["ClusterConfigurationTypeDef"],
     },
-    total=False,
 )
 
 SubnetGroupTypeDef = TypedDict(
     "SubnetGroupTypeDef",
     {
-        "Name": str,
-        "Description": str,
-        "VpcId": str,
-        "Subnets": List["SubnetTypeDef"],
-        "ARN": str,
+        "Name": NotRequired[str],
+        "Description": NotRequired[str],
+        "VpcId": NotRequired[str],
+        "Subnets": NotRequired[List["SubnetTypeDef"]],
+        "ARN": NotRequired[str],
     },
-    total=False,
 )
 
 SubnetTypeDef = TypedDict(
     "SubnetTypeDef",
     {
-        "Identifier": str,
-        "AvailabilityZone": "AvailabilityZoneTypeDef",
+        "Identifier": NotRequired[str],
+        "AvailabilityZone": NotRequired["AvailabilityZoneTypeDef"],
     },
-    total=False,
 )
 
 TagResourceRequestRequestTypeDef = TypedDict(
@@ -1171,20 +987,18 @@ TagResourceResponseTypeDef = TypedDict(
 TagTypeDef = TypedDict(
     "TagTypeDef",
     {
-        "Key": str,
-        "Value": str,
+        "Key": NotRequired[str],
+        "Value": NotRequired[str],
     },
-    total=False,
 )
 
 UnprocessedClusterTypeDef = TypedDict(
     "UnprocessedClusterTypeDef",
     {
-        "ClusterName": str,
-        "ErrorType": str,
-        "ErrorMessage": str,
+        "ClusterName": NotRequired[str],
+        "ErrorType": NotRequired[str],
+        "ErrorMessage": NotRequired[str],
     },
-    total=False,
 )
 
 UntagResourceRequestRequestTypeDef = TypedDict(
@@ -1203,27 +1017,14 @@ UntagResourceResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredUpdateACLRequestRequestTypeDef = TypedDict(
-    "_RequiredUpdateACLRequestRequestTypeDef",
+UpdateACLRequestRequestTypeDef = TypedDict(
+    "UpdateACLRequestRequestTypeDef",
     {
         "ACLName": str,
+        "UserNamesToAdd": NotRequired[Sequence[str]],
+        "UserNamesToRemove": NotRequired[Sequence[str]],
     },
 )
-_OptionalUpdateACLRequestRequestTypeDef = TypedDict(
-    "_OptionalUpdateACLRequestRequestTypeDef",
-    {
-        "UserNamesToAdd": Sequence[str],
-        "UserNamesToRemove": Sequence[str],
-    },
-    total=False,
-)
-
-
-class UpdateACLRequestRequestTypeDef(
-    _RequiredUpdateACLRequestRequestTypeDef, _OptionalUpdateACLRequestRequestTypeDef
-):
-    pass
-
 
 UpdateACLResponseTypeDef = TypedDict(
     "UpdateACLResponseTypeDef",
@@ -1233,38 +1034,25 @@ UpdateACLResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredUpdateClusterRequestRequestTypeDef = TypedDict(
-    "_RequiredUpdateClusterRequestRequestTypeDef",
+UpdateClusterRequestRequestTypeDef = TypedDict(
+    "UpdateClusterRequestRequestTypeDef",
     {
         "ClusterName": str,
+        "Description": NotRequired[str],
+        "SecurityGroupIds": NotRequired[Sequence[str]],
+        "MaintenanceWindow": NotRequired[str],
+        "SnsTopicArn": NotRequired[str],
+        "SnsTopicStatus": NotRequired[str],
+        "ParameterGroupName": NotRequired[str],
+        "SnapshotWindow": NotRequired[str],
+        "SnapshotRetentionLimit": NotRequired[int],
+        "NodeType": NotRequired[str],
+        "EngineVersion": NotRequired[str],
+        "ReplicaConfiguration": NotRequired["ReplicaConfigurationRequestTypeDef"],
+        "ShardConfiguration": NotRequired["ShardConfigurationRequestTypeDef"],
+        "ACLName": NotRequired[str],
     },
 )
-_OptionalUpdateClusterRequestRequestTypeDef = TypedDict(
-    "_OptionalUpdateClusterRequestRequestTypeDef",
-    {
-        "Description": str,
-        "SecurityGroupIds": Sequence[str],
-        "MaintenanceWindow": str,
-        "SnsTopicArn": str,
-        "SnsTopicStatus": str,
-        "ParameterGroupName": str,
-        "SnapshotWindow": str,
-        "SnapshotRetentionLimit": int,
-        "NodeType": str,
-        "EngineVersion": str,
-        "ReplicaConfiguration": "ReplicaConfigurationRequestTypeDef",
-        "ShardConfiguration": "ShardConfigurationRequestTypeDef",
-        "ACLName": str,
-    },
-    total=False,
-)
-
-
-class UpdateClusterRequestRequestTypeDef(
-    _RequiredUpdateClusterRequestRequestTypeDef, _OptionalUpdateClusterRequestRequestTypeDef
-):
-    pass
-
 
 UpdateClusterResponseTypeDef = TypedDict(
     "UpdateClusterResponseTypeDef",
@@ -1290,27 +1078,14 @@ UpdateParameterGroupResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredUpdateSubnetGroupRequestRequestTypeDef = TypedDict(
-    "_RequiredUpdateSubnetGroupRequestRequestTypeDef",
+UpdateSubnetGroupRequestRequestTypeDef = TypedDict(
+    "UpdateSubnetGroupRequestRequestTypeDef",
     {
         "SubnetGroupName": str,
+        "Description": NotRequired[str],
+        "SubnetIds": NotRequired[Sequence[str]],
     },
 )
-_OptionalUpdateSubnetGroupRequestRequestTypeDef = TypedDict(
-    "_OptionalUpdateSubnetGroupRequestRequestTypeDef",
-    {
-        "Description": str,
-        "SubnetIds": Sequence[str],
-    },
-    total=False,
-)
-
-
-class UpdateSubnetGroupRequestRequestTypeDef(
-    _RequiredUpdateSubnetGroupRequestRequestTypeDef, _OptionalUpdateSubnetGroupRequestRequestTypeDef
-):
-    pass
-
 
 UpdateSubnetGroupResponseTypeDef = TypedDict(
     "UpdateSubnetGroupResponseTypeDef",
@@ -1320,27 +1095,14 @@ UpdateSubnetGroupResponseTypeDef = TypedDict(
     },
 )
 
-_RequiredUpdateUserRequestRequestTypeDef = TypedDict(
-    "_RequiredUpdateUserRequestRequestTypeDef",
+UpdateUserRequestRequestTypeDef = TypedDict(
+    "UpdateUserRequestRequestTypeDef",
     {
         "UserName": str,
+        "AuthenticationMode": NotRequired["AuthenticationModeTypeDef"],
+        "AccessString": NotRequired[str],
     },
 )
-_OptionalUpdateUserRequestRequestTypeDef = TypedDict(
-    "_OptionalUpdateUserRequestRequestTypeDef",
-    {
-        "AuthenticationMode": "AuthenticationModeTypeDef",
-        "AccessString": str,
-    },
-    total=False,
-)
-
-
-class UpdateUserRequestRequestTypeDef(
-    _RequiredUpdateUserRequestRequestTypeDef, _OptionalUpdateUserRequestRequestTypeDef
-):
-    pass
-
 
 UpdateUserResponseTypeDef = TypedDict(
     "UpdateUserResponseTypeDef",
@@ -1353,13 +1115,12 @@ UpdateUserResponseTypeDef = TypedDict(
 UserTypeDef = TypedDict(
     "UserTypeDef",
     {
-        "Name": str,
-        "Status": str,
-        "AccessString": str,
-        "ACLNames": List[str],
-        "MinimumEngineVersion": str,
-        "Authentication": "AuthenticationTypeDef",
-        "ARN": str,
+        "Name": NotRequired[str],
+        "Status": NotRequired[str],
+        "AccessString": NotRequired[str],
+        "ACLNames": NotRequired[List[str]],
+        "MinimumEngineVersion": NotRequired[str],
+        "Authentication": NotRequired["AuthenticationTypeDef"],
+        "ARN": NotRequired[str],
     },
-    total=False,
 )
